package Classes;

import Interface.JFPrincipal;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.ArrayList;

public class ClienteThread extends Thread implements Processo {

    private Socket conexao;
    private JFPrincipal pai;
    private ObjectOutputStream saida;
    private ObjectOutputStream saidaChat;
    private ObjectInputStream entrada;
    private String nome;
    private String nome2;
    private ArrayList<String> cartas;
    private String coringa;
    private String ipServidor;
    private int porta;
    private int jogadorAcao;
    private String cartaAcao;
    private int rodadas[];
    private int pontoPartida;
    private boolean onTruco;            //  Flag que define se jogador já está em truco ou não.
    private ChatReceber chat;
    
    public ClienteThread(JFPrincipal pai, String nome, String ipServidor, int porta) {
        this.pai = pai;
        this.nome = nome;
        this.ipServidor = ipServidor;
        this.porta = porta;
        this.cartas = new ArrayList<String>();
        jogadorAcao = 0;
        int a[] = {-1, -1, -1};
        this.rodadas = a;
        this.onTruco = false;
    }

    public void enviarString(String msg) {

        try {
            saida.writeObject(msg);
        } catch (IOException ex) {
            System.err.println("Erro ao enviar msg: " + ex.toString());
        }
    }

    @Override
    public String getCoringa() {
        return coringa;
    }    
    
    @Override
    public void enviar(int codigo, String cont) throws Exception {        
        Mensagem msg = new Mensagem(codigo, cont);                
        saida.writeObject(msg);
        System.out.println("Enviando mensagem para o cliente: " + msg.getCodigo() + " + " + msg.getConteudo());
    }    

    @Override
    public void enviarChat(int codigo, String cont) throws Exception {        
        Mensagem msg = new Mensagem(codigo, cont);                
        saidaChat.writeObject(msg);
        System.out.println("Enviando mensagem para o cliente: " + msg.getCodigo() + " + " + msg.getConteudo());
    }        
    
    @Override
    public int qualRodada(int quemLevou[]) {
        
        int i = 2;     
        //  Descobre em qual rodada o jogo está.
        for(; i > 0; i--) {
            if (quemLevou[i] > -1) {
                break;
            }
        }
        return i;
    }    
    
    @Override
    public Mensagem receber() throws IOException, ClassNotFoundException {
        return (Mensagem) entrada.readObject();
    }
    
    @Override
    public int[] getRodadas() {
        return rodadas;
    }    
    
    @Override
    public void run() {        

        try {
            conexao = new Socket(ipServidor, porta);
            saida = new ObjectOutputStream(conexao.getOutputStream());
            entrada = new ObjectInputStream(conexao.getInputStream());           
            
            //  Primeira mensagem. Entrar no Jogo.
            enviar(100, nome);
            
            //  Passa referência deste processo para o processo pai.
            pai.setaProcesso((Processo)this);
            
            pai.mensagem("Conexão bem sucedida!\n Jogo será iniciado...");
            
            boolean flag = true;
            do{
                tratarMensagem(receber());
            }while(flag);
            
        } catch (Exception ex) {
            pai.mensagem("Conexão falhou!");
            System.err.println("Erro sockets: " + ex.toString());
        }
    }    
    
    public boolean tratarMensagem(Mensagem mensagem) throws Exception {
        
        int codigo = mensagem.getCodigo();
        
        switch(codigo) {
            
            //  Handshaking. Entrar no jogo.
            case 100:
                pai.alerta(mensagem.getConteudo() + " entrou no jogo!");
                this.nome2 = mensagem.getConteudo();
                pai.setPlacar(0, nome2, 0);
                enviar(103, "");                                         
                return true;
            //  Receber Cartas
            case 200:
                this.setarMao(mensagem.getConteudo());
                return true;                
            //  Recebe solicitação de carta e responde.
            case 201:
                String resp = this.pedirCartaCliente();
                enviar(201, resp);
                if(resp.equals("l")) {
                    pai.alerta("Você largou sua mão de cartas.");
                }
                return true;
                
            //  Receber carta da jogada.
            case 202:
                pai.exibirCartaJogada(mensagem.getConteudo());
                pai.limparUmaCartaAdver();
                return true;
            //  Jogador servidor escondeu.
            case 203:
                pai.alerta(nome2 + " escondeu a carta!");
                pai.exibirCartaJogada(mensagem.getConteudo());
                return true;                
                //  Jogador tentou esconder no momento errado.
            case 204:
                pai.alerta(nome2 + " tentou esconder a carta.");
                return true;                 
                
            case 205:                
                String m[] = mensagem.getConteudo().split(":");
                rodadas[0] = Integer.parseInt(m[0]);
                rodadas[1] = Integer.parseInt(m[1]);
                rodadas[2] = Integer.parseInt(m[2]);
                pai.mensagem("Recebendo rodadas: " + rodadas[0] + " " + rodadas[1] + " " + rodadas[2]);                
                pai.mensagem("recebeu atualização de rodadas");
                return true;  
                
                //  O outro jogador largou a mão
            case 206:
                pai.alerta(nome2 + " largou a mão.");
                pai.limpaCartasJogadas();
                return true;
                
                //  Trucar.
            case 207:
                this.tratarTruco(mensagem.getConteudo());
                return true;
                
                //  Truco aceito.
            case 208:
                this.pontoPartida = Integer.parseInt(mensagem.getConteudo());
                pai.alerta("Truco aceito! Escolha uma carta.");
                return true;
                
            //  Jogador que venceu a rodada.
            case 300:
                pai.alerta(mensagem.getConteudo() + " venceu a rodada!");
                pai.limpaCartasJogadas();
                return true;
                
                //  Atualização da pontuação.
            case 301:
                this.setPlacar(mensagem.getConteudo());                
                return true;
                
                //  Rodada empatou.
            case 302:
                pai.alerta("A Rodada empatou!");
                pai.limpaCartasJogadas();                
                return true;
                
                //  Jogador que venceu a partida.
            case 303:
                pai.alerta(mensagem.getConteudo() + " venceu a partida!");
                return true;            
                
                //  Resetar flag onTruco.
            case 304:
                this.onTruco = false;
                return true;
                //  Atualiza valor da partida.
            case 305:
                this.pontoPartida = Integer.parseInt(mensagem.getConteudo());
                return true;    
                //  Setar mão de onze true.
            case 306:
                pai.setMaoDeOnze(true);
                return true;
                //  Setar mão de onze false.
            case 307:
                pai.setMaoDeOnze(false);
                return true;
                //  Receber Mensagem do Chat.                
            case 666:                
                pai.enviarChat(mensagem.getConteudo());
                return true;
                //  Mensagem Simples
            case 900:
            pai.alerta(mensagem.getConteudo());
                return true;
                //  Terminar jogo.
            case 901:
                pai.alerta(nome2 + " desconectou.");
                saida.close();
                entrada.close();
                conexao.close();
                return false;
                //  Gambiarra de campeão. Cliente retorna o conteúdo da mensagem.
            case 999:
                this.enviar(999, mensagem.getConteudo());
                return true;            
        }
        return true;
    }    

    public String pedirCartaCliente() throws Exception {
        
        pai.solicitaCarta();
        
        Mensagem msg;
        
        //  Gambiarra de campeão.
        do{
            msg = this.receber();
            if(msg.getCodigo() == 666) {
                pai.enviarChat(msg.getConteudo());
            }
        }while(msg.getCodigo() != 999);
        //  O jogador anfitrião enviara do procesos GUI uma mensagem 
        // com um código(?) que o clente retornará o conteúdo.
        switch(Integer.parseInt(msg.getConteudo())) {
            
            //  Jogada normal.
            case 1:
                pai.exibirCartaJogada(cartaAcao);                      
                return cartaAcao;                
            //  Esconder.
            case 2:
                pai.exibirCartaJogada("fundoa");
                return "0o";    //  Valor mais fraco do jogo. Valor de controle.
                //  Largar mão.
            case 3:
                pai.limpaCartasJogadas();
                return "l";
                //  Jogador servidor largou a mão.        
            case 4:
                return "Tr";
            default:
                System.out.println("Default");
                break;
        }
        return "";
    }    
  
    public void setPlacar(String msg) {
        
        String[] pont = msg.split(":");        
        pai.setPlacar(Integer.parseInt(pont[0]), nome2, Integer.parseInt(pont[1]));        
    }
    
    @Override
    public int getPontoPartida() {
        return pontoPartida;
    }        
    
    public void setarMao(String msg) {

      this.cartas.clear();
      
            String[] mao = msg.split(":");

            //  Seta mão do jogador
            for(int i = 0; i < 3; i++) {
                this.cartas.add(mao[i]);
            }
            coringa = mao[3];

            //  Seta coringa da rodada
            coringa  = mao[3];
            pai.exibirMao(cartas, coringa);
    }

    @Override
    public void setaJogadorAcao(int acao) {
        try {
            this.enviar(999, "" + acao);        //  1 - Escolheu uma carta; 2 - Largou mão; 3 - Trucou. 4 - Truco recusado
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
    }
    
    @Override
    public void setaCartaAcao(String carta) {
        this.cartaAcao = carta;
    }

    @Override
    public int condEsconder(int quemLevou[]) {
        
        int i = 2;     
        //  Descobre em qual rodada o jogo está.
        for(; i > 0; i--) {
            if (quemLevou[i] > -1) {
                break;
            }
        }
        //  Se o primeiro turno já foi decidido.
        if((i == 0)&&(quemLevou[i] != -1)) {
            return 1;
        }
        
        return i;                
    }
    
    public void tratarTruco(String ti) throws Exception {
        int r = pai.trucoResposta(ti);
        this.enviar(r, "");
    }
    
    @Override
    public void setOnTruco(boolean a) {
        this.onTruco = a;
    }
    @Override
    public boolean getOnTruco() {
        return onTruco;
    }

    @Override
    public void incPontoPartida(int valor) {
        pontoPartida += valor;
    }
    
}
