package Classes;

import Interface.JFPrincipal;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ServidorThread extends Thread implements Processo{

    private Socket conexao;
    private int porta;
    private JFPrincipal pai;
    private ServerSocket servidor;
    private ObjectOutputStream saida;
    private ObjectOutputStream saidaChat;
    private ObjectInputStream entrada;
    private Mensagem mensagem;
    private String coringa;
    private ArrayList<Jogador> jogadores;
    private boolean ftermina;
    private int jogadorQCaiPrimeiro;
    private int jogadorAcao;            //  1 - Jogar carta. 2 - Esconder.
    private String cartaAcao;
    private int jogadorRodada;
    private int rodadas[];                      //  Para poder descobrir a rodada de fora da classe.
    private int pontoPartida;           //  Pontuação disputada na partida. Inicialmente = 1
    private boolean onTruco;            //  Flag que define se jogador já está em truco ou não.
    private ChatReceber chat;
    
    
    public ServidorThread(int porta, JFPrincipal pai) throws IOException {
        this.porta = porta;
        this.pai = pai;
        servidor = new ServerSocket(porta);
        this.mensagem = new Mensagem();
        this.jogadores = new ArrayList<Jogador>();
        this.ftermina = true;
        this.jogadorAcao = 0;
        this.onTruco = false;
    }

    @Override
    public int[] getRodadas() {
        return rodadas;
    }
    
    public String getIp() {
        return servidor.getInetAddress().toString();
    }

    @Override
    public String getCoringa() {
        return coringa;
    }
    
    @Override
    public void enviar(int codigo, String cont) throws Exception {        
        Mensagem msg = new Mensagem(codigo, cont);                
        saida.writeObject(msg);
        System.out.println("Enviando mensagem para o cliente: " + msg.getCodigo() + " + " + msg.getConteudo());
    }    

    @Override
    public void enviarChat(int codigo, String cont) throws Exception {        
        Mensagem msg = new Mensagem(codigo, cont);                
        saidaChat.writeObject(msg);
        System.out.println("Enviando mensagem para o cliente: " + msg.getCodigo() + " + " + msg.getConteudo());
    }      
    
    @Override
    public Mensagem receber() throws IOException, ClassNotFoundException {
        return (Mensagem) entrada.readObject();
    }

    @Override
    public void run() {
        try {
            conexao = servidor.accept();
            saida = new ObjectOutputStream(conexao.getOutputStream());
            entrada = new ObjectInputStream(conexao.getInputStream());
            
            //  Cria jogador 1;
            JogadorHumano p1 = new JogadorHumano(pai.getNick());
            jogadores.add(p1);
            
            //  Handshaking. Enviar nick do jogador.
            this.enviar(100, jogadores.get(0).getNome());

            //  Passa referência deste processo para o processo pai.
            pai.setaProcesso((Processo)this);
            
            //  Limpa mesa.
            pai.limpaTudoNaMesa();
            
            boolean flag = true;
            do {
                flag = tratarMensagem(receber());
            } while (flag);

        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
    }

    public boolean tratarMensagem(Mensagem mensagem) throws Exception {

        int codigo = mensagem.getCodigo();

        switch (codigo) {

            // Entrar no Jogo
            case 100:
                //  Criar Jogador 2.
                JogadorHumano p2 = new JogadorHumano(mensagem.getConteudo());               
                jogadores.add(p2);  //  Adicionar jogador ao array.

                //  Atualizar placar (colocar os nomes dos jogadores).
                pai.setPlacar(jogadores.get(0).getPontuacao(), jogadores.get(1).getNome(), jogadores.get(1).getPontuacao());
                pai.alerta(p2.getNome() + " entrou no jogo!");                
                return true;
        
                // Jogador Cliente confirma início de jogo.
            case 103:
                this.iniciarJogo();
                int esc = 0;
                
                do {
                    esc = pai.trucoResposta("Reiniciar jogo?");
                    if(esc == 0) {
                        this.iniciarJogo();
                    }
                    else {
                        this.enviar(901,"");
                        entrada.close();
                        saida.close();                        
                        conexao.close();                        
                    }
                } while(esc == 0);
                
                return true;
                
                //  Jogador tentou esconder a carta.
            case 204:
                pai.alerta(jogadores.get(1).getNome() + " tentou esconder a carta.");
                return true; 
        }
        return true;
    }

    public void iniciarJogo() throws Exception {

        jogadorQCaiPrimeiro = sorteiaJogadorQCai(); //  Sorteia jogador que caira primeiro.       
        jogadores.get(0).setPontuacao(0);
        jogadores.get(1).setPontuacao(0);
        this.enviar(301, "" + jogadores.get(1).getPontuacao() + ":" + jogadores.get(0).getPontuacao()); //  Atualiza Placar do cliente.
        pai.setMaoDeOnze(false);
        this.enviar(307, "");   //  Setar mão de onze para false no cliente.
        
        pai.setPlacar(jogadores.get(0).getPontuacao(), jogadores.get(1).getNome(), jogadores.get(1).getPontuacao());
        
        do {
            //rodada, partida, jogo.
            this.iniciaPartida();
        } while (testaVitoria() && ftermina); // O loop termina quando algum jogador alcançar 12 pontos ou o flag de segurança for setado

        String ms = "Fim de jogo! ";
        if(jogadores.get(0).getPontuacao() > jogadores.get(1).getPontuacao()) {
            ms += jogadores.get(0).getNome() + " Venceu o Jogo!";
        } else {
            ms += jogadores.get(1).getNome() + " Venceu o Jogo!";
        }
        pai.alerta(ms);
        this.enviar(900, ms);
    }

    public void iniciaPartida() throws Exception {
        
        //  Array de cartas. Na verdade só vai segurar dois valores por vez.
        String[] cartas = {"", ""};
        
        //  Seta pontuação inicial.
        pontoPartida = 1;
        enviar(305, "" + getPontoPartida());
        
        int fLargarMao = -1;  //  0 - jogador 1, 1 jogador 2;
        
        int quemLevou[] = {-1, -1, -1};      // -1 - Rodada não jogada; 5 - Empate; 0 - P1 levou; 1 - P2 levou;        
        
        //  Resetar flags onTruco.
        this.onTruco = false;
        this.enviar(304, "");
        
        //  Para tratar caso de empate na mão de onze.
        if((jogadores.get(0).getPontuacao() == jogadores.get(1).getPontuacao() &&
                jogadores.get(0).getPontuacao() == 11) ) {
            pai.setMaoDeOnze(true);
            this.enviar(306, "");   //  Setar mão de onze no cliente.
        } 
        
        distribuiMao();
        
        pai.exibirMao(jogadores.get(0).getCartas(), coringa);        
        jogadorRodada = jogadorQCaiPrimeiro;    //  Copia jogador que inicia a partida para a var de jogo local.

            do {
                rodadas = quemLevou;
                this.enviar(205, "" + rodadas[0] + ":" + rodadas[1] + ":" + rodadas[2]);
                pai.mensagem("Enviado rodadas: " + rodadas[0] + " " + rodadas[1] + " " + rodadas[2]);

                if (jogadorRodada == 0) {
                    cartas[0] = pedirCartaServer();  //  Pede carta para o server.
                    if (cartas[0].equals("l")) {    //  Jogador Anfitrião largou a mão.
                        pai.mensagem("Largou a mão - IniciaPartida()");
                        fLargarMao = 0;
                        break;
                    } else if (cartas[0].equals("Tr")) { //    Segundo jogador recusou o truco.
                        quemLevou[0] = 0;
                        quemLevou[1] = 0;
                        quemLevou[2] = 0;
                        break;
                    }
                    cartas[1] = pedirCartaClient();  //  Pede carta para o cliente.               
                    if (cartas[1].equals("l")) {     //  Jogador Convidado largou a mão.
                        fLargarMao = 1;
                        break;
                    } else if (cartas[1].equals("Tr")) { //    Segundo jogador recusou o truco.
                        quemLevou[0] = 1;
                        quemLevou[1] = 1;
                        quemLevou[2] = 1;
                        break;
                    }

                    quemLevou = calculaRodada(cartas[0], cartas[1], quemLevou);
                    int a = informaJogador(quemLevou);
                    if (a != 5) {
                        jogadorRodada = a;
                    }
                } else {
                    cartas[1] = pedirCartaClient();  //  Pede carta para o cliente.               
                    if (cartas[1].equals("l")) {     //  Jogador Convidado largou a mão.
                        fLargarMao = 1;
                        break;
                    } else if (cartas[1].equals("Tr")) { //    Segundo jogador recusou o truco.
                        quemLevou[0] = 1;
                        quemLevou[1] = 1;
                        quemLevou[2] = 1;
                        break;
                    }
                    cartas[0] = pedirCartaServer();  //  Pede carta para o server.
                    if (cartas[0].equals("l")) {    //  Jogador Anfitrião largou a mão.
                        pai.mensagem("Largou a mão - IniciaPartida()");
                        fLargarMao = 0;
                        break;
                    } else if (cartas[0].equals("Tr")) { //    Segundo jogador recusou o truco.
                        quemLevou[0] = 0;
                        quemLevou[1] = 0;
                        quemLevou[2] = 0;
                        break;
                    }

                    quemLevou = calculaRodada(cartas[0], cartas[1], quemLevou);
                    int a = informaJogador(quemLevou);
                    if (a != 5) {
                        jogadorRodada = a;
                    }
                }
                pai.limpaCartasJogadas();

            } while ((testaVencedorRodada(quemLevou) == -1) && (fLargarMao < 0));

        if(fLargarMao > -1) {        
            if(fLargarMao == 0) {    //  Jogador servidor escondeu.
            //  Incrementa pontuação.
            jogadores.get(1).incPontuacao(pontoPartida);                
            //  Avisa jogador servidor sobre vencedor da partida.
            pai.alerta(jogadores.get(1).getNome() + " venceu a partida!");
            //  Atualiza placar.
            pai.setPlacar(jogadores.get(0).getPontuacao(), jogadores.get(1).getNome(), jogadores.get(1).getPontuacao());
            //  Avisa jogador cliente sobre vencedor da partida.
            enviar(303, jogadores.get(1).getNome());
            //  Atualiza placar do cliente.
            enviar(301, "" + jogadores.get(1).getPontuacao() + ":" + jogadores.get(0).getPontuacao());                
            }
            else {  //  Jogador cliente escondeu.
            //  Incrementa pontuação.
            jogadores.get(0).incPontuacao(pontoPartida);                
            //  Avisa jogador servidor sobre vencedor da partida.
            pai.alerta(jogadores.get(0).getNome() + " venceu a partida!");
            //  Atualiza placar.
            pai.setPlacar(jogadores.get(0).getPontuacao(), jogadores.get(1).getNome(), jogadores.get(1).getPontuacao());
            //  Avisa jogador cliente sobre vencedor da partida.
            enviar(303, jogadores.get(0).getNome());
            //  Atualiza placar do cliente.
            enviar(301, "" + jogadores.get(1).getPontuacao() + ":" + jogadores.get(0).getPontuacao());                  
            }
            pai.limpaCartasJogadas();
        }
        else {
            int a = testaVencedorRodada(quemLevou);
            //  Incrementa pontuação.
            jogadores.get(a).incPontuacao(pontoPartida);
            //  Avisa jogador servidor sobre vencedor da partida.
            pai.alerta(jogadores.get(a).getNome() + " venceu a partida!");
            //  Atualiza placar.
            pai.setPlacar(jogadores.get(0).getPontuacao(), jogadores.get(1).getNome(), jogadores.get(1).getPontuacao());
            //  Avisa jogador cliente sobre vencedor da partida.
            enviar(303, jogadores.get(a).getNome());
            //  Atualiza placar do cliente.
            enviar(301, "" + jogadores.get(1).getPontuacao() + ":" + jogadores.get(0).getPontuacao());
        }
        //  Se jogador que caiu agora é o 1 e o flag de empate não está true.
        if(jogadorQCaiPrimeiro == 0) {
            jogadorQCaiPrimeiro = 1;
        }
                
        //  Se jogador que caiu agora é o 2 e o flag de empate não está true.                
        else if(jogadorQCaiPrimeiro == 1) {
            jogadorQCaiPrimeiro = 0;
        }
    }
    
    @Override
    public void incPontoPartida(int valor) {
        pontoPartida += valor;
    }
    @Override
    public int getPontoPartida() {
        return pontoPartida;
    }    
    //  Retorna pontuação da rodada.
    public int[] calculaRodada(String cartaP1, String cartaP2, int quemLevou[]) {        
        
        int p1 = valorDaCarta(cartaP1);
        int p2 = valorDaCarta(cartaP2);
        
        //  Primeira Rodada
        if (quemLevou[0] == -1) {
            
            if (p1 > p2) {          //  Jogador 1 vence.
                quemLevou[0] = 0;                
            } 
            else if (p2 > p1) {     //  Jogador 2 vence.
                quemLevou[0] = 1;
            } 
            else if (p1 == p2) {    //  Empatou a primeira.
                quemLevou[0] = 5;
            }
        }
        //  Segunda Rodada
        else if (quemLevou[1] == -1) {
            
            if (p1 > p2) {          //  Jogador 1 vence.
                quemLevou[1] = 0;                
            } 
            else if (p2 > p1) {     //  Jogador 2 vence.
                quemLevou[1] = 1;
            } 
            else if (p1 == p2) {    //  Empatou a segunda.
                
                if(quemLevou[0] != 5) {
                    
                    if(quemLevou[0] == 0) {     //  Quem venceu a primeira, vence partida.
                        quemLevou[1] = 0;
                    }
                    else {
                        quemLevou[1] = 1;
                    }                    
                }
                else {              //  Empatou a segunda.  quemLevou[0] == 5 - true
                    quemLevou[1] = 5;
                }                
            }            
        } 
        //  Terceira Rodada       
        else if (quemLevou[2] == -1) {
            
            if (p1 > p2) {          //  Jogador 1 vence.
                quemLevou[2] = 0;                
            } 
            else if (p2 > p1) {     //  Jogador 2 vence.
                quemLevou[2] = 1;
            } 
            else if (p1 == p2) {    //  Empatou a terceira.                
                if(quemLevou[0] != 5) {
                    
                    if(quemLevou[0] == 0) {     //  Quem venceu a primeira, vence partida.
                        quemLevou[2] = 0;
                    }
                    else {
                        quemLevou[2] = 1;
                    }                    
                }
                else if(quemLevou[1] != 5) {    //  Quem levou a segunda vence.
                    
                    if(quemLevou[1] == 0) {
                        quemLevou[2] = 0;
                    }
                    else {
                        quemLevou[2] = 1;
                    }   
                }
                else {                      //  Se empatou a primeira e a segunda, a carta de maior nipe vence.
                    if(valorDoNipe(cartaP1) > valorDoNipe(cartaP2)) {
                        quemLevou[2] = 0;
                    }
                    else {
                        quemLevou[2] = 1;
                    }
                }
            }            
        }
        return quemLevou;
    }

    //  Retorna jogador vencedor ou -1 se ainda não foi possível decidir.
    public int testaVencedorRodada(int quemLevou[]) {        
        
        int p[] = {0, 0};
        
        //  Controle. Começa a testar apenas a partir da segunda rodada.
        if(quemLevou[1] == -1) {
            return -1;
        }
        //  Se alguém venceu a primeira.
        if(quemLevou[0] != 5) {
            p[quemLevou[0]]++;      //  Adiciona um ponto.
            
            if((quemLevou[1] != 5)&&(quemLevou[1] != quemLevou[0])) { //  Se alguém venceu a segunda rodada e não foi o mesmo jogador da primeira.
                p[quemLevou[1]]++;  //  Adiciona um ponto.
                
                if(quemLevou[2] != -1) {
                    p[quemLevou[2]]++;
                }
            }
            else {                  //  Se não quem venceu a primeira rodada, vence a partida.
                return quemLevou[0];
            }
        }                
        //  Se não se alguém venceu a segunda.
        else if (quemLevou[1] != 5) {
            return quemLevou[1];      //    Vencedor da segunda rodada vence.
        }        
        else if(quemLevou[2] != -1) {   //  Se a terceira rodada foi jogada.
            return quemLevou[2];        //  Se não quem venceu a terceira rodada ganha o ponto.
        }
        else {
            return -1;
        }
        
        //  Se jogador 1 fez mais pontos que jogador 2, vence jogador 1.
        if(p[0] > p[1]) {
            return 0;
        }
        else if (p[1] > p[0]){  //  Se não vence jogador 2.
            return 1;
        }
        else {
            return -1;
        }
    }    
   
    public int informaJogador(int quemLevou[]) throws Exception {  

        int i = qualRodada(quemLevou);
        
        if (quemLevou[i] != 5) {
            enviar(300, jogadores.get(quemLevou[i]).getNome());            
            pai.alerta(jogadores.get(quemLevou[i]).getNome() + " venceu a rodada!");
            return quemLevou[i];
        } 
        else {
            enviar(302, "");            
            pai.alerta("A rodada empatou!");
            return 5;
        }
    }
    
    @Override
    public int qualRodada(int quemLevou[]) {
        
        int i = 2;     
        //  Descobre em qual rodada o jogo está.
        for(; i > 0; i--) {
            if (quemLevou[i] > -1) {
                break;
            }
        }
        return i;
    }
    
    @Override
    public int condEsconder(int quemLevou[]) {
        
        int i = 2;             //  Descobre em qual rodada o jogo está.
        for(; i > 0; i--) {
            if (quemLevou[i] > -1) {
                break;
            }
        }
        //  Se o primeiro turno já foi decidido.
        if((i == 0)&&(quemLevou[i] != -1)) {
            return 1;
        }
        
        return i;                
    }
    
    public String pedirCartaServer() throws Exception {
        
        pai.solicitaCarta();       
        
        Mensagem msg;
        
        //  Gambiarra de campeão.
        do{
            msg = this.receber();
            if(msg.getCodigo() == 666) {
                pai.enviarChat(msg.getConteudo());
            }
        }while(msg.getCodigo() != 999);
        //  O jogador anfitrião enviara do procesos GUI uma mensagem 
        // com um código(?) que o clente retornará o conteúdo.
        switch(Integer.parseInt(msg.getConteudo())) {
            
                //  Jogada normal.
            case 1:
                enviar(202, cartaAcao);    //  Enviar carta para o outro jogador.
                pai.exibirCartaJogada(cartaAcao);
                return cartaAcao;
                //  Esconder.
            case 2:
                enviar(203, "0o");
                pai.exibirCartaJogada("0o");
                return "0o";
                //  Largar mão.
            case 3:
                pai.mensagem("Largou a mão-pedirCartaServer()");
                enviar(206, "");
                return "l";                
                //  Caso pedido de truco foi recusado.
            case 4:
                return "Tr";
                
            default:
                System.out.println("Default");
                break;
        }
        return "";
    }
    
    public String pedirCartaClient() throws Exception {
        
        this.enviar(201, "");   //  Pedir carta para jogador.       
        
        //  A resposta para uma mensagem 201 sempre será outra mensagem 201, com a resposta. MENTIRA.
        Mensagem msg;
        do{
            msg = receber();
            if(msg.getCodigo() == 204) {
                pai.alerta(jogadores.get(1).getNome() + " tentou esconder a carta.");
            }
                    //  Cliente Trucou
            else if (msg.getCodigo() == 207) {
                switch(pai.trucoResposta("TRUCO!")) {
                    case 0:
                        if (pontoPartida >= 3) {     //  Se partida já estiver trucada.
                            incPontoPartida(3);
                        } else {                        //  Se não > incremento inicial.
                            incPontoPartida(2);
                        } 
                        enviar(0, "");                        
                        break;
                        //  Recusou.
                    case 1:
                        enviar(1,"");
                        break;
                }
            } else if(msg.getCodigo() == 666) {
                pai.enviarChat(msg.getConteudo());
            }
                    //  Parte da gambiarra de campeão.
            else if (msg.getCodigo() == 999) {
                this.enviar(999, msg.getConteudo());
            }
        }while(msg.getCodigo() != 201);
        
        //sif(msg.getCodigo() != 201) System.err.println("Erro ao receber carta do cliente.");      
        if(msg.getConteudo().equals("Tr")){
        } else if(msg.getConteudo().equals("l")){
        } else {
            //  Para o caso de ser a escura. (diria que é gambiarra, mas estou começando a crer que tudo fora do plano inicial se torna gambiarra =D)
            pai.exibirCartaJogada(msg.getConteudo());
            pai.limparUmaCartaAdver();
        }
        return msg.getConteudo();
    }
    
    @Override
    public void setaJogadorAcao(int acao) {
        try {
            this.enviar(999, "" + acao);        //  1 - Escolheu uma carta; 2 - Largou mão; 3 - Trucou. 4 - Truco recusado
        } catch (Exception ex) {
            Logger.getLogger(ServidorThread.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    @Override
    public void setaCartaAcao(String carta) {
        this.cartaAcao = carta;
    }
    
    public int sorteiaJogadorQCai() {
        return (int)(2*Math.random());
    }
    
    //  Distribui Mão e tira coringa.
    public void distribuiMao() throws Exception {

        //  Embaralha as cartas.
        pai.embaralhar();
        
        //  Limpar tudo na Mesa.  >gambiarra
        //pai.limpaTudoNaMesa();
        
        //  Remover mão dos jogadores.
        jogadores.get(0).limparCartas();
        jogadores.get(1).limparCartas();

        // Retira duas mãos de cartas e o coringa
        ArrayList<String> cartas = pai.pegaCarta(7);

        //  Seta cartas do jogador 1.
        for (int i = 0; i < 6; i++) {
            if (i < 3) {
                jogadores.get(0).addCartas(cartas.get(i));
            } else {
                jogadores.get(1).addCartas(cartas.get(i));
            }
        }    
        
        coringa = cartas.get(6);

        /*  //Controle e testes.
        jogadores.get(0).limparCartas();
        jogadores.get(1).limparCartas();
        
        jogadores.get(0).addCartas("do");
        jogadores.get(0).addCartas("7o");
        jogadores.get(0).addCartas("jo");
        
        jogadores.get(1).addCartas("2p");
        jogadores.get(1).addCartas("3p");
        jogadores.get(1).addCartas("4p");
        
        coringa = "7c";
        */
        
        //  Envia cartas para o cliente
        String m = new String();
        for (String c : jogadores.get(1).getCartas()) {
            m += c;
            m += ":";
        }
        m += coringa;
       
        enviar(200, m);
    }

    public boolean testaVitoria() {
        if( (jogadores.get(0).getPontuacao() >= 12) || (jogadores.get(1).getPontuacao() >= 12)) {
            return false;
        }
        return true;
    }
    
    public int valorDaCarta(String cartaP) {
        
        char[] p = {cartaP.charAt(0), cartaP.charAt(1)};
        
        int C1 = 0;
        
        char corV = coringa.charAt(0);
        char corN = coringa.charAt(1);
               
        //  Se a carta virada para o coringa for 7.
        if(corV == '7') {   //  Mais gambiarra.
            corV = 'd';     //  Que beleza que e java. Tratar char como char.
        } else {
            corV++;         //  Tratar char como inteiro. D:D
        }
        pai.mensagem("Coringa é " + corV);  //  Controle.
        
        if(p[0] == corV) {
            
            if(cartaP.charAt(1) == 'o')
                C1 = 11;
            if(cartaP.charAt(1) == 'e')
                C1 = 12;
            if(cartaP.charAt(1) == 'c')
                C1 = 13;
            if(cartaP.charAt(1) == 'p')
                C1 = 14;                        
        }
        else {            
            if(cartaP.charAt(0) == '4')
                C1 = 1;
            if(cartaP.charAt(0) == '5')
                C1 = 2;
            if(cartaP.charAt(0) == '6')
                C1 = 3;
            if(cartaP.charAt(0) == '7')
                C1 = 4;
            if(cartaP.charAt(0) == 'd')
                C1 = 5;
            if(cartaP.charAt(0) == 'j')
                C1 = 6;
            if(cartaP.charAt(0) == 'q')
                C1 = 7;
            if(cartaP.charAt(0) == '1')
                C1 = 8;
            if(cartaP.charAt(0) == '2')
                C1 = 9;
            if(cartaP.charAt(0) == '3')
                C1 = 10;        
            if(cartaP.charAt(0) == '0')
                C1 = 0;              
        }
        return C1;
    }    
    
    public int valorDoNipe(String cartaP) {           

        if (cartaP.charAt(1) == 'o') {      //  Ouro
            return 0;
        }
        else if (cartaP.charAt(1) == 'e') { //  Espadas
            return 1;
        }
        else if (cartaP.charAt(1) == 'c') { //  Copas
            return 2;
        }
        else {                              //  Paus
            return 3;
        }
    }
    
    @Override
    public void setOnTruco(boolean a) {
        this.onTruco = a;
    }
    @Override
    public boolean getOnTruco() {
        return onTruco;
    }     
}
